#include <windows.h>
#include "draw.h"

const double PI = 3.141592653;

LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
char szClassName[] = "CG_DDA";

////////////////////////////////////////////////////////////////////////////////////////////////////
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	HWND hWnd;  
	MSG lpMsg;
	WNDCLASS wc;
	
	//    
	wc.style         = CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc   = WndProc;
	wc.cbClsExtra	 = 0;
	wc.cbWndExtra    = 0;
	wc.hInstance     = hInstance;
	wc.hIcon		 = NULL;
	wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
	wc.lpszMenuName  = NULL;
	wc.lpszClassName = szClassName;

	//   
	if (!RegisterClass(&wc))
	{
		MessageBox(NULL, "Cannot register class", "Error", MB_OK);
		return 0;
	}
	
	//    
	hWnd = CreateWindow( 
		szClassName,		    // Class name                   
		"CG7 - DDA Line",       // Text for window title bar                  
		WS_OVERLAPPEDWINDOW,    // Window style            						
		50,	50,                 // Use default position  
		600,  600,              // Width and height    
		(HWND) NULL,            // Overlapped windows have no parent    
		(HMENU) NULL,           // Use the window class menu              
		(HINSTANCE)hInstance,   // This instance owns this window              
		NULL    );				// We don't use any data in our WM_CREATE       
	
	if (!hWnd) 
	{
		MessageBox(NULL, "Cannot create main window", "Error", MB_OK);
		return 0;
	}

	//   
	ShowWindow(hWnd, nCmdShow); 
	UpdateWindow(hWnd);

	//       
	while (GetMessage(&lpMsg, NULL, 0, 0))  {
		TranslateMessage(&lpMsg);
		DispatchMessage(&lpMsg);
	}

	return (lpMsg.wParam);
}

////////////////////////////////////////////////////////////////////////////////////////////////////
LRESULT CALLBACK WndProc(HWND hWnd, UINT messg, WPARAM wParam, LPARAM lParam)
{
	PAINTSTRUCT ps;
	RECT Rect;
	HDC hdc, hCmpDC;
	HBITMAP hBmp;
	static int x1, y1, x2, y2;
	static bool draw_line = false;

	switch (messg)
	{
		case WM_PAINT:
			GetClientRect(hWnd, &Rect);
			hdc = BeginPaint(hWnd, &ps);
			//      
			hCmpDC = CreateCompatibleDC(hdc);
			hBmp = CreateCompatibleBitmap(hdc, Rect.right - Rect.left,
				Rect.bottom - Rect.top);
			SelectObject(hCmpDC, hBmp);
			//   
			LOGBRUSH br;
			br.lbStyle = BS_SOLID;
			br.lbColor = 0xEECCCC;
			HBRUSH brush;
			brush = CreateBrushIndirect(&br);
			FillRect(hCmpDC, &Rect, brush);
			DeleteObject(brush);
			// 
			if (draw_line)
			{
				line_DDA(hCmpDC, x1, y1, x2, y2);
			}
			//   
			SetStretchBltMode(hdc, COLORONCOLOR);
			BitBlt(hdc, 0, 0, Rect.right - Rect.left, Rect.bottom - Rect.top,
				hCmpDC, 0, 0, SRCCOPY);
			DeleteDC(hCmpDC);
			DeleteObject(hBmp);
			hCmpDC = NULL;
			EndPaint(hWnd, &ps);
			break;

		case WM_LBUTTONDOWN:
			x1 = LOWORD(lParam);
			y1 = HIWORD(lParam);
			draw_line = false;
			InvalidateRect(hWnd, NULL, FALSE);
			break;

		case WM_MOUSEMOVE:
			if (UINT(wParam) & MK_LBUTTON) {
				x2 = LOWORD(lParam);
				y2 = HIWORD(lParam);
				draw_line = true;
				InvalidateRect(hWnd, NULL, FALSE);
			}
			break;

		case WM_LBUTTONUP:
			draw_line = true;
			break;

		case WM_DESTROY:
			PostQuitMessage(0);
			break;

		default:
			return (DefWindowProc(hWnd, messg, wParam, lParam));
	}

	return (0);
}
